@@ -2,7 +2,7 @@
  * The SIP library code that implements the interface to the optional module
  * supplied Qt support.
  *
- * Copyright (c) 2020 Riverbank Computing Limited <info@riverbankcomputing.com>
+ * Copyright (c) 2011 Riverbank Computing Limited <info@riverbankcomputing.com>
  *
  * This file is part of SIP.
  *
@@ -22,6 +22,7 @@
 #include <assert.h>
 #include <string.h>
 
+#include "sip.h"
 #include "sipint.h"
 
 
@@ -71,11 +72,7 @@ static void *newSignal(void *txrx, const char **sig)
 static void *createUniversalSlot(sipWrapper *txSelf, const char *sig,
         PyObject *rxObj, const char *slot, const char **member, int flags)
 {
-    void *us;
-
-    assert(sipQtSupport->qt_create_universal_slot);
-
-    us = sipQtSupport->qt_create_universal_slot(txSelf, sig, rxObj, slot,
+    void *us = sipQtSupport->qt_create_universal_slot(txSelf, sig, rxObj, slot,
             member, flags);
 
     if (us && txSelf)
@@ -86,30 +83,16 @@ static void *createUniversalSlot(sipWrapper *txSelf, const char *sig,
 
 
 /*
- * Invoke a single slot (Qt or Python) and return the result.  Don't check if
- * any receiver C++ object still exists.
+ * Invoke a single slot (Qt or Python) and return the result.
  */
 PyObject *sip_api_invoke_slot(const sipSlot *slot, PyObject *sigargs)
-{
-    return sip_api_invoke_slot_ex(slot, sigargs, TRUE);
-}
-
-
-/*
- * Invoke a single slot (Qt or Python) and return the result.  Optionally check
- * that any receiver C++ object still exist.
- */
-PyObject *sip_api_invoke_slot_ex(const sipSlot *slot, PyObject *sigargs,
-        int no_receiver_check)
 {
     PyObject *sa, *oxtype, *oxvalue, *oxtb, *sfunc, *sref;
 
-    assert(sipQtSupport);
-
     /* Keep some compilers quiet. */
     oxtype = oxvalue = oxtb = NULL;
 
-    /* Fan out Qt signals.  (Only PyQt3 would do this.) */
+    /* Fan out Qt signals.  (Only PyQt3 will do this.) */
     if (slot->name != NULL && slot->name[0] != '\0')
     {
         assert(sipQtSupport->qt_emit_signal);
@@ -159,8 +142,7 @@ PyObject *sip_api_invoke_slot_ex(const sipSlot *slot, PyObject *sigargs,
          * If the receiver wraps a C++ object then ignore the call if it no
          * longer exists.
          */
-        if (!no_receiver_check &&
-            PyObject_TypeCheck(self, (PyTypeObject *)&sipSimpleWrapper_Type) &&
+        if (PyObject_TypeCheck(self, (PyTypeObject *)&sipSimpleWrapper_Type) &&
             sip_api_get_address((sipSimpleWrapper *)self) == NULL)
         {
             Py_XDECREF(sref);
@@ -218,7 +200,7 @@ PyObject *sip_api_invoke_slot_ex(const sipSlot *slot, PyObject *sigargs,
     {
         PyObject *nsa, *xtype, *xvalue, *xtb, *resobj;
 
-        if ((resobj = PyObject_CallObject(sfunc, sa)) != NULL)
+        if ((resobj = PyEval_CallObject(sfunc, sa)) != NULL)
         {
             Py_DECREF(sfunc);
             Py_XDECREF(sref);
@@ -325,9 +307,6 @@ PyObject *sip_api_invoke_slot_ex(const sipSlot *slot, PyObject *sigargs,
  */
 int sip_api_same_slot(const sipSlot *sp, PyObject *rxObj, const char *slot)
 {
-    assert(sipQtSupport);
-    assert(sipQtSupport->qt_same_name);
-
     /* See if they are signals or Qt slots, ie. they have a name. */
     if (slot != NULL)
     {
@@ -372,9 +351,6 @@ int sip_api_same_slot(const sipSlot *sp, PyObject *rxObj, const char *slot)
 void *sipGetRx(sipSimpleWrapper *txSelf, const char *sigargs, PyObject *rxObj,
            const char *slot, const char **memberp)
 {
-    assert(sipQtSupport);
-    assert(sipQtSupport->qt_find_slot);
-
     if (slot != NULL)
         if (isQtSlot(slot) || isQtSignal(slot))
         {
@@ -407,8 +383,6 @@ void *sipGetRx(sipSimpleWrapper *txSelf, const char *sigargs, PyObject *rxObj,
 void *sip_api_convert_rx(sipWrapper *txSelf, const char *sigargs,
         PyObject *rxObj, const char *slot, const char **memberp, int flags)
 {
-    assert(sipQtSupport);
-
     if (slot == NULL)
         return createUniversalSlot(txSelf, sigargs, rxObj, NULL, memberp, flags);
 
@@ -439,9 +413,6 @@ void *sip_api_convert_rx(sipWrapper *txSelf, const char *sigargs,
 PyObject *sip_api_connect_rx(PyObject *txObj, const char *sig, PyObject *rxObj,
         const char *slot, int type)
 {
-    assert(sipQtSupport);
-    assert(sipQtSupport->qt_connect);
-
     /* Handle Qt signals. */
     if (isQtSignal(sig))
     {
@@ -465,7 +436,7 @@ PyObject *sip_api_connect_rx(PyObject *txObj, const char *sig, PyObject *rxObj,
         return PyBool_FromLong(res);
     }
 
-    /* Handle Python signals.  Only PyQt3 would get this far. */
+    /* Handle Python signals.  Only PyQt3 will get this far. */
     assert(sipQtSupport->qt_connect_py_signal);
 
     if (sipQtSupport->qt_connect_py_signal(txObj, sig, rxObj, slot) < 0)
@@ -482,10 +453,6 @@ PyObject *sip_api_connect_rx(PyObject *txObj, const char *sig, PyObject *rxObj,
 PyObject *sip_api_disconnect_rx(PyObject *txObj,const char *sig,
                 PyObject *rxObj,const char *slot)
 {
-    assert(sipQtSupport);
-    assert(sipQtSupport->qt_disconnect);
-    assert(sipQtSupport->qt_destroy_universal_slot);
-
     /* Handle Qt signals. */
     if (isQtSignal(sig))
     {
@@ -518,7 +485,7 @@ PyObject *sip_api_disconnect_rx(PyObject *txObj,const char *sig,
         return PyBool_FromLong(res);
     }
 
-    /* Handle Python signals.  Only PyQt3 would get this far. */
+    /* Handle Python signals.  Only PyQt3 will get this far. */
     assert(sipQtSupport->qt_disconnect_py_signal);
 
     sipQtSupport->qt_disconnect_py_signal(txObj, sig, rxObj, slot);
@@ -533,8 +500,6 @@ PyObject *sip_api_disconnect_rx(PyObject *txObj,const char *sig,
  */
 void sip_api_free_sipslot(sipSlot *slot)
 {
-    assert(sipQtSupport);
-
     if (slot->name != NULL)
     {
         sip_api_free(slot->name);
@@ -570,8 +535,6 @@ static char *sipStrdup(const char *s)
  */
 int sip_api_save_slot(sipSlot *sp, PyObject *rxObj, const char *slot)
 {
-    assert(sipQtSupport);
-
     sp -> weakSlot = NULL;
 
     if (slot == NULL)
